'Propeller Chip MV4th16 Timer COG demo 
'
CON
  _clkmode = xtal1 + pll16x
  _xinfreq = 5_000_000

PUB Start
  ''does nothing but must be there for spin IDE to compile
  
DAT
              org       0
Entry         mov       TimeAdr, PAR
              mov       LockAdr, PAR
              add       LockAdr, #4
              mov       TimeDelayAdr, PAR
              add       TimeDelayAdr, #8
              mov       LastTime, #0
              call      #ClrLock
              'Start time timer
              rdlong    TimeDelay, TimeDelayAdr
              mov       LoopWait, cnt
              add       LoopWait, TimeDelay
loop          waitcnt   LoopWait, TimeDelay
              'Ready to update the time
              'See if time was set
              call      #SetLock
              rdlong    T1, TimeAdr
              cmp       T1, LastTime            wz
              mov       LastTime, T1
        if_nz call      #GetTime
              call      #IncTime
              call      #SetTime
              call      #ClrLock
              'if we loaded a new time reset counter
        if_nz mov       LoopWait, cnt
        if_nz add       LoopWait, TimeDelay
              jmp       #loop
'
'This function sets the lock bit for the time data
SetLock       rdlong    T1, LockAdr
SetLock1      lockset   T1                      wc
        if_c  jmp       #SetLock1
SetLock_ret   ret
'                      
'This function clears the lock bit for the time data
ClrLock       rdlong    T1, LockAdr
              lockclr   T1
ClrLock_ret   ret
'                      
'This function reads the 32 bit BCD packet time value and unpacks
'the time value into the Hour, Minute, Second, SecHund variables
GetTime       rdlong    T1, TimeAdr
              'Get the hundreths of a second value.
              mov       SecHund, T1
              and       SecHund, #$FF
              'Get the seconds value.
              shr       T1, #8
              mov       Second, T1
              and       Second, #$FF
              'Get the minutes value.
              shr       T1, #8
              mov       Minute, T1
              and       Minute, #$FF
              'Get the hours value.
              shr       T1, #8
              mov       Hour, T1
GetTime_ret   ret
'
'
'This function builds a 32 bit BCD packed time. The 32 bit BCD
'if formed by packing the time values Hour, Minute, Second, SecHund. The
'result is saved to the 32 bit global memory address of the system time.
              'Set the hundreths of a second value.
SetTime       mov       T1, SecHund
              'Set the seconds value.
              mov       T2, Second
              shl       T2, #8
              or        T1, T2
              'Set the minutes value.
              mov       T2, Minute
              shl       T2, #16
              or        T1, T2
              'Set the hours value.
              mov       T2, Hour
              shl       T2, #24
              or        T1, T2
              wrlong    T1, TimeAdr
SetTime_ret   ret
'
'
'This function increments the two digit BCD value in T1 (lower 8 bits)
BcdInc        add       T1, #1
              mov       T2, T1
              and       T2, #$0F
              cmp       T2, #$0A                 wc
        if_c  jmp       #BcdInc_ret                    
              'have a carry to tens digit
              and       T1, #$F0                'clear first digit
              add       T1, #$10                'increment tens digit
BcdInc_ret    ret
'
'
'This function adds 1 hundreth of a second to the time saved in the
'variables Hour, Minute, Second, and SecHund.
IncTime       mov       T1, SecHund
              call      #BcdInc
              mov       SecHund, T1
              cmp       SecHund, #$9A           wc
        if_c  jmp       #IncTime_ret
              'Hundreths of a seconds overflow 
              mov       SecHund, #0
              'increment seconds
              mov       T1, Second
              call      #BcdInc
              mov       Second, T1
              cmp       Second, #$5A             wc
        if_c  jmp       #IncTime_ret
              'Seconds overflow
              mov       Second, #0
              'increment minutes
              mov       T1, Minute
              call      #BcdInc
              mov       Minute, T1
              cmp       Minute, #$5A             wc
        if_c  jmp       #IncTime_ret
              'Minutes overflow
              mov       Minute, #0
              'increment hours
              mov       T1, Hour
              call      #BcdInc
              mov       Hour, T1
              cmp       Hour, #$24               wc
        if_c  jmp       #IncTime_ret
              'Minutes overflow
              mov       Hour, #0
IncTime_ret   ret
'
'
'Constats start here
TimeAdr       long      0
LockAdr       long      0
TimeDelayAdr  long      0          
TimeDelay     long      800_000    '80mhz CPU clock ferquency divided by 100 for example
'Variables start here
Hour          res       1
Minute        res       1
Second        res       1
SecHund       res       1
LoopWait      res       1
T1            res       1
T2            res       1
LastTime      res       1